local _, L = ...
local mediaFolder = "Interface\\AddOns\\".._.."\\Media\\"
local LMB, RMB = CreateAtlasMarkup("NPE_LeftClick", 18, 18), CreateAtlasMarkup("NPE_RightClick", 18,18)

menuPanelMixin = {}

--[[ Nameplate side button bar ]]--
local function ChangeMenuPanelSize(self)
	if not InCombatLockdown() then
		self:SetWidth(IsInGroup() and 89 or 39)
	else
		C_Timer.After(2, function()	ChangeMenuPanelSize(self); end)
	end
end

function menuPanelMixin:OnGroupChange()
	if IsInRaid() then
		if self.group ~= "raid" then
			self.group = "raid"
			RaidOrParty:SetNormalTexture(mediaFolder.."icons\\party")
			RaidOrParty:SetPushedTexture(mediaFolder.."icons\\party")
			RaidOrParty.tooltipText = LMB.." - convert raid to party"
			RaidOrParty:SetHighlightTexture(mediaFolder.."icons\\party", "ADD")
		end
	elseif self.group ~= "party" then
		self.group = "party"
		RaidOrParty:SetNormalTexture(mediaFolder.."icons\\raid")
		RaidOrParty:SetPushedTexture(mediaFolder.."icons\\raid")
		RaidOrParty.tooltipText = LMB.." - convert party to raid"
		RaidOrParty:SetHighlightTexture(mediaFolder.."icons\\raid", "ADD")
	end
					
	if ToggleButton.state == true then
		ChangeMenuPanelSize(self)
	end
end
			
function menuPanelMixin:CreateMenuPanel()
	local MARKERS = {5, 6, 3, 2, 7, 1, 4, 8}
	local button, previousButton
	local buttonSize = 24
	local toggleSize = 7
	local frameAlpha = 0.8
	
	local function ButtonOnEnter(self)
		GameTooltip:SetOwner(self, "ANCHOR_TOP")
		GameTooltip:AddLine(self.tooltipText, 0, 1, 0.5, false)
		GameTooltip:Show()
	end

	local function ButtonOnLeave(self)
		GameTooltip:Hide()
	end
	
	menuPanel:SetPoint("LEFT", UIParent, "LEFT", -1, 0)
	menuPanel:SetAlpha(frameAlpha)
	menuPanel:SetBackdrop({bgFile = "Interface\\Buttons\\WHITE8x8", edgeFile = "Interface\\Buttons\\WHITE8x8", tile = false, tileSize = 1, edgeSize = 1, insets = { left = 1, right = 1, top = 1, bottom = 1 },})
	menuPanel:SetBackdropColor(0, 0, 0, frameAlpha)
	menuPanel:SetBackdropBorderColor(1, 0, 0)
	menuPanel:SetSize(toggleSize + 2, 162)
	menuPanel:SetMovable(true)
	
	function menuPanel:CreateMenuButton(name, enabled, size, normalTexture, pushedTexture, tooltipText)
		local button = CreateFrame("Button", name, self, "SecureActionButtonTemplate")
			button:SetSize(size, size)
			button.tooltipText = tooltipText
			button:SetNormalTexture(normalTexture)
			button:SetPushedTexture(pushedTexture)
			if enabled then
				button:SetButtonState("PUSHED")
			end
			button:SetHighlightTexture(normalTexture, "ADD")
			button:SetScript("OnEnter", ButtonOnEnter)
			button:SetScript("OnLeave", ButtonOnLeave)
			button:RegisterForClicks("LeftButtonUp", "RightButtonUp") --AnyDown"
		return button
	end
	
	-- prvy stlpec
	for i = 1, #MARKERS do
		button = menuPanel:CreateMenuButton("mpWMButton"..i, nil, 16, mediaFolder.."marks\\"..i, mediaFolder.."marks\\"..i, LMB.." - put worldmark"..i.."\n"..RMB.." - remove worldmark"..i)
		button:SetAttribute("type", "macro")
		button:SetAttribute("macrotext", "/wm [btn:1] "..MARKERS[i].."\n/cwm [btn:2] "..MARKERS[i])
		
		if not previousButton then
			button:SetPoint("TOPRIGHT", menuPanel, -71, -5)
		else
			button:SetPoint("TOP", previousButton, "BOTTOM", 0, -1)
		end
		previousButton = button
	end
	
	button = menuPanel:CreateMenuButton("Button9", nil, 17, "Interface\\Buttons\\UI-GroupLoot-Pass-Up", "Interface\\Buttons\\UI-GroupLoot-Pass-Up", LMB.." - remove all worldmarks")
	button:SetPoint("TOP", previousButton, "BOTTOM", 0, -1)
	button:SetAttribute("type", "macro")
	button:SetAttribute("macrotext", "/cwm all")
	
	-- druhy stlpec
	button = menuPanel:CreateMenuButton("RaidOrParty", nil, buttonSize, IsInRaid() and mediaFolder.."icons\\party" or mediaFolder.."icons\\raid", IsInRaid() and mediaFolder.."icons\\party" or mediaFolder.."icons\\raid", LMB.." - convert party to raid")
	button:SetPoint("TOPRIGHT", menuPanel, -41, -5)
	button:SetScript("OnClick", function(self, button)
		if IsInRaid() then
			C_PartyInfo.ConvertToParty()
		elseif IsInGroup() then
			C_PartyInfo.ConvertToRaid()
		else
			print("|cff00bbffYou|r |cff00bbff are not in|r |cff00ff00group|r |cff00bbffor|r |cff00ff00party|r.")
		end
	end)
	previousButton = button
	
	StaticPopupDialogs["LEAVE_GROUP"] = {
		text = "Please confirm to leave your group.\n\n",
		button1 = "Yes",
		button2 = "Yes and thanks",
		button3 = "No",
		OnAccept = function()
			C_PartyInfo.LeaveParty()
		end,
		OnCancel = function()
			SendChatMessage('Thanks for the group, ppl!' , namePlateMixin:InstanceOrParty())
			C_Timer.After(1, function()	C_PartyInfo.LeaveParty(); end)
		end,
		OnAlt = function ()
		end,
		timeout = 0,
		whileDead = true,
		showAlert = true,
		exclusive = true,
		hideOnEscape = true,
		preferredIndex = 3,
	}
	StaticPopupDialogs["KICK_GROUP"] = {
		text = "%s",
		button1 = "Write to party",
		button2 = CANCEL,
		OnAccept = function()
			SendChatMessage("Please kick this marked NPC:" , namePlateMixin:InstanceOrParty())
			for guid in pairs(namePlateMixin.PARTY_TABLE) do
				SendChatMessage("{rt"..L.config.mythic_kicker_mark[namePlateMixin.PARTY_TABLE[guid].mark].."} "..namePlateMixin.PARTY_TABLE[guid].name  , namePlateMixin:InstanceOrParty())
			end
		end,
		OnCancel = function()
		end,
		timeout = 0,
		whileDead = true,
		showAlert = false,
		exclusive = true,
		hideOnEscape = true,
		preferredIndex = 3,
	}
						
	button = menuPanel:CreateMenuButton("LeaveGroup", nil, buttonSize, mediaFolder.."icons\\leave", mediaFolder.."icons\\leave", LMB.." - leave group\n")
	button:SetPoint("TOP", previousButton, "BOTTOM", 0, -1)
	button:SetScript("OnClick", function(self, button)
		if button ==  "LeftButton" or button == "RightButton"then
			StaticPopup_Show("LEAVE_GROUP")
		end		
	end)
	previousButton = button
	
	button = menuPanel:CreateMenuButton("ReadyCheck", nil, buttonSize, "Interface\\RaidFrame\\ReadyCheck-Ready", "Interface\\RaidFrame\\ReadyCheck-Ready", LMB.." - ready check\n"..RMB.." - show kick priority")
	button:SetPoint("TOP", previousButton, "BOTTOM", 0, -1)
	button:SetScript("OnClick", function(self, button)
		if button ==  "LeftButton" then
			DoReadyCheck()
		elseif button == "RightButton"then
			local t = ""
			for guid in pairs(namePlateMixin.PARTY_TABLE) do
				t = t .. ICON_LIST[L.config.mythic_kicker_mark[namePlateMixin.PARTY_TABLE[guid].mark]].."16:16|t "..namePlateMixin.PARTY_TABLE[guid].name.."\n"
			end
			StaticPopup_Show("KICK_GROUP", t)
		end
	end)
	previousButton = button
	
	button = menuPanel:CreateMenuButton("Pull10", nil, buttonSize, mediaFolder.."icons\\10", mediaFolder.."icons\\10", LMB.." - start countdown 10 sec\n"..RMB.." - stop active countdown")
	button:SetPoint("TOP", previousButton, "BOTTOM", 0, -1)
	button:SetScript("OnClick", function(self, button)
		if button ==  "LeftButton" then
			C_PartyInfo.DoCountdown(10)
		elseif button == "RightButton"then
			SendChatMessage('Countdown is canceled!!!' , namePlateMixin:InstanceOrParty())
			C_PartyInfo.DoCountdown(0)
		end
	end)
	previousButton = button
	
	button = menuPanel:CreateMenuButton("Pull3", nil, buttonSize, mediaFolder.."icons\\3", mediaFolder.."icons\\3", LMB.." - start countdown 3 sec\n"..RMB.." - stop active countdown")
	button:SetPoint("TOP", previousButton, "BOTTOM", 0, -1)
	button:SetScript("OnClick", function(self, button)
		if button ==  "LeftButton" then
			C_PartyInfo.DoCountdown(3)
		elseif button == "RightButton"then
			SendChatMessage('Countdown is canceled!!!' , namePlateMixin:InstanceOrParty())
			C_PartyInfo.DoCountdown(0)
		end
	end)
	previousButton = button

	button = menuPanel:CreateMenuButton("Keys", nil, buttonSize, mediaFolder.."icons\\keys", mediaFolder.."icons\\keys", LMB.." - /!keys to guild chat\n"..RMB.." - /!keys to party chat")
	button:SetPoint("TOP", previousButton, "BOTTOM", 0, -1)
	button:SetAttribute("type", "macro")
	button:SetAttribute("macrotext", "/run if SecureCmdOptionParse'[btn:1]' then SendChatMessage('!keys' ,'GUILD') elseif SecureCmdOptionParse'[btn:2]' then SendChatMessage('!keys' ,'PARTY')  end")
	previousButton = button
	
	-- treti stlpec
	button = menuPanel:CreateMenuButton("ButtonMarks", nil, buttonSize, mediaFolder.."icons\\target", mediaFolder.."icons\\target", LMB.." - add or remove your target to M+ Database\n"..RMB.." - open config for setup this nameplates")
	button:SetPoint("TOPRIGHT", menuPanel, -11, -5)
	button:SetScript("OnClick", function(self, button)
		local namePlate = C_NamePlate.GetNamePlateForUnit("target", issecure())
		if namePlate and not UnitIsPlayer("target") then
			if button ==  "LeftButton" then
				if not namePlate.UnitFrame.mapID then 
					namePlate.UnitFrame.mapID = 0
				end
					
				if not L.MythicNPCData[namePlate.UnitFrame.mapID][namePlate.UnitFrame.npcID] then
					L.MythicNPCData[namePlate.UnitFrame.mapID][namePlate.UnitFrame.npcID] = {}
					L.MythicNPCData[namePlate.UnitFrame.mapID][namePlate.UnitFrame.npcID].destName = namePlate.UnitFrame.unitName
					
					L.UpdateNPCDatabase(namePlate.UnitFrame.mapID, namePlate.UnitFrame.npcID, nil)
					LibStub("AceConfigRegistry-3.0"):NotifyChange("Nameplates")
					namePlateMixin:UpdateNamePlateByOptions("nameplate")
					namePlateMixin:DebugLog("|cff00bbffYou|r |cff00ff00added|r |cff00ff80"..namePlate.UnitFrame.unitName.."|r|cff00bbff to M+ Database.|r", 3)
				else
					namePlateMixin:DebugLog("|cff00bbffYou|r |cffff0000removed|r |cff00ff80"..namePlate.UnitFrame.unitName.."|r|cff00bbff from M+ Database.|r", 3)
					L.MythicNPCData[namePlate.UnitFrame.mapID][namePlate.UnitFrame.npcID] = nil
					L.UpdateNPCDatabase(0, 0, nil)
					LibStub("AceConfigRegistry-3.0"):NotifyChange("Nameplates")
					namePlateMixin:UpdateNamePlateByOptions("nameplate")
				end
				--namePlateMixin:UpdateNamePlateByOptions(namePlate.UnitFrame)
		
			elseif button == "RightButton" then 
				if namePlate.UnitFrame.mapID and namePlate.UnitFrame.npcID then 
					LibStub('AceConfigDialog-3.0'):SelectGroup("Nameplates", "NPC_DB")
					LibStub('AceConfigDialog-3.0'):Open("Nameplates")
					L.UpdateNPCDatabase(namePlate.UnitFrame.mapID, namePlate.UnitFrame.npcID, nil)
				else
					namePlateMixin:DebugLog("|cffFF0000Your target dont exist in database.|r", 3)
				end
			end		
		else
			namePlateMixin:DebugLog("|cffFF0000You do not have a correct target.|r", 3)
		end
	end)
	
	previousButton = button
	
	button = menuPanel:CreateMenuButton("ButtonHP", L.config.hp_enable, buttonSize, mediaFolder.."icons\\hp", mediaFolder.."icons\\hp_e", LMB.." - enable or disable health points\n"..RMB.." - open config for setup")
	button:SetPoint("TOP", previousButton, "BOTTOM", 0, -1)
	button:SetScript("OnClick", function(self, button)
		if button ==  "LeftButton" then
			if L.config.hp_enable then
				L.config.hp_enable = false
			else
				L.config.hp_enable = true
			end
			self:SetButtonState("PUSHED", L.config.hp_enable)
			LibStub("AceConfigRegistry-3.0"):NotifyChange("Nameplates")
			print(L.config.hp_enable and "|cff00ff00ENABLE|r" or "|cffff0000DISABLE|r", "|cff00bbffhealth points value on nameplate|r ")
			namePlateMixin:UpdateNamePlateByOptions("hp_enable")
		elseif button == "RightButton" then
			LibStub('AceConfigDialog-3.0'):SelectGroup("Nameplates", "health_power_setup")
			LibStub('AceConfigDialog-3.0'):Open("Nameplates")
		end
	end)
	previousButton = button
	
	button = menuPanel:CreateMenuButton("ButtonValue", L.config.mythic_percent_show, buttonSize, mediaFolder.."icons\\percenta", mediaFolder.."icons\\percenta_e", "NPCs value on nameplates in M+ ")
	button:SetPoint("TOP", previousButton, "BOTTOM", 0, -1)
	button:SetScript("OnClick", function(self, button)
		if button == "LeftButton" then
			if ( L.config.mythic_plus_enable ) then
				if L.config.mythic_percent_show then
					L.config.mythic_percent_show = false
				else
					L.config.mythic_percent_show = true
				end
				self:SetButtonState("PUSHED", L.config.mythic_percent_show)
				LibStub("AceConfigRegistry-3.0"):NotifyChange("Nameplates")
				print(L.config.mythic_percent_show and "|cff00ff00ENABLE|r" or "|cffff0000DISABLE|r", "|cff00bbffM+ value on nameplate|r ")
				namePlateMixin:UpdateNamePlateByOptions("mythic")
			else
				print("|cff00bbffMythic plus extension is disabled.|r")
			end
		end
	end)
	previousButton = button

	button = menuPanel:CreateMenuButton("ButtonRange", L.config.show_range, buttonSize, mediaFolder.."icons\\distance", mediaFolder.."icons\\distance_e", LMB.." - enable or disable distance\n"..RMB.." - open config for setup")
	button:SetPoint("TOP", previousButton, "BOTTOM", 0, -1)
	button:SetScript("OnClick", function(self, button)
		if button ==  "LeftButton" then
			if L.config.show_range then
				L.config.show_range = false
			else
				L.config.show_range = true
			end
			self:SetButtonState("PUSHED", L.config.show_range)
			LibStub("AceConfigRegistry-3.0"):NotifyChange("Nameplates")
			print(L.config.show_range and "|cff00ff00ENABLE|r" or "|cffff0000DISABLE|r", "|cff00bbffdistannce on|r", L.config.range_only_on_target and "|cff00ff00target |cff00bbffnameplate.|r" or "|cff00ff00all |cff00bbffnameplates.|r")
			namePlateMixin:UpdateNamePlateByOptions("show_range")
						
		elseif button == "RightButton" then
			LibStub('AceConfigDialog-3.0'):SelectGroup("Nameplates", "extension")
			LibStub('AceConfigDialog-3.0'):Open("Nameplates")
		end	
	end)
	previousButton = button
	
	button = menuPanel:CreateMenuButton("ButtonAutoMark", L.config.auto_mark, buttonSize, mediaFolder.."icons\\mark", mediaFolder.."icons\\mark_e", LMB.." - enable or disable auto marking\n"..RMB.." - open config for setup")
	button:SetPoint("TOP", previousButton, "BOTTOM", 0, -1)
	button:SetScript("OnClick", function(self, button)
		if button ==  "LeftButton" then
			if L.config.auto_mark then
				L.config.auto_mark = false
			else
				L.config.auto_mark = true
			end
			self:SetButtonState("PUSHED", L.config.auto_mark)
			LibStub("AceConfigRegistry-3.0"):NotifyChange("Nameplates")
			print(L.config.auto_mark and "|cff00ff00ENABLE|r" or "|cffff0000DISABLE|r", "|cff00bbff auto marking in M+ Dungeons.|r")
			namePlateMixin:UpdateNamePlateByOptions("auto_mark")
			
		elseif button == "RightButton" then
			LibStub('AceConfigDialog-3.0'):SelectGroup("Nameplates", "extension", "raidmarks")
			LibStub('AceConfigDialog-3.0'):Open("Nameplates")
		end
	end)
	previousButton = button

	button = menuPanel:CreateMenuButton("ButtonConfig", nil, buttonSize, mediaFolder.."icons\\config", mediaFolder.."icons\\config", LMB.." - addon configuration\n"..RMB.." - open target NPC in M+ database")
	button:SetPoint("TOP", previousButton, "BOTTOM", 0, -1)
	button:SetScript("OnClick", function(self, button)
		if button ==  "LeftButton" then
			LibStub('AceConfigDialog-3.0'):SelectGroup("Nameplates", "basic_page")
			LibStub('AceConfigDialog-3.0'):Open("Nameplates")
		elseif button == "RightButton" then
			LibStub('AceConfigDialog-3.0'):SelectGroup("Nameplates", "NPC_DB")
			LibStub('AceConfigDialog-3.0'):Open("Nameplates")
			
			local namePlate = C_NamePlate.GetNamePlateForUnit("target", issecure())
			if namePlate and not UnitIsPlayer("target") then
				if not namePlate.UnitFrame.mapID then 
					namePlate.UnitFrame.mapID = 0
				end
				
				if namePlate.UnitFrame.mapID and namePlate.UnitFrame.npcID then 
					L.UpdateNPCDatabase(namePlate.UnitFrame.mapID, namePlate.UnitFrame.npcID, nil)
				end
			end
		end
	end)
	previousButton = button
	
	-- bocna lista
	local toggleButton = CreateFrame("BUTTON", "ToggleButton", menuPanel, "SecureHandlerClickTemplate")
	toggleButton:SetPoint("TOPRIGHT", -2, -1)
	toggleButton:SetPoint("BOTTOMRIGHT", -2, 1)
	toggleButton:SetWidth(toggleSize)
	toggleButton:RegisterForClicks("LeftButtonUp")
	
	toggleButton.bg = toggleButton:CreateTexture(nil, "BACKGROUND", nil)
	toggleButton.bg:SetAllPoints()
	toggleButton.bg:SetColorTexture(0, 0, 0)
	toggleButton.bg:SetAlpha(0.4)
	
	toggleButton.tooltipText = LMB.." - open or close menuPanel\n"..RMB.." - move menuPanel to new position"
	toggleButton:SetScript("OnEnter", function(self)
		ButtonOnEnter(self)
		toggleButton.bg:SetAlpha(1)
	end)
	toggleButton:SetScript("OnLeave", function(self)
		ButtonOnLeave(self)
		toggleButton.bg:SetAlpha(.4)
	end)
	
	toggleButton:EnableMouse(true)
	toggleButton:SetClampedToScreen(true)
	toggleButton:RegisterForDrag("RightButton")
    
	toggleButton:SetScript("OnMouseDown", function(self, button)
		if button == "LeftButton" then
			if not self.state then
				self.state = false
			end
			if self.state == false then
				if IsInGroup() then
					self:GetParent():SetWidth(89)
				else
					self:GetParent():SetWidth(39)
				end
				self.state = true
			else
				self:GetParent():SetWidth(7 + 2)
				self.state = false
			end
			
		elseif button == "RightButton" and not InCombatLockdown() then
			self:GetParent():SetClampedToScreen(true)
			self:GetParent():StartMoving()
		end
	end)
    
	toggleButton:SetScript("OnMouseUp", function(self, button)
		if button == "RightButton" and not InCombatLockdown() then
			local cursorX, cursorY = GetCursorPosition()
			self:GetParent():SetClampedToScreen(false)
			self:GetParent():StopMovingOrSizing()
			self:GetParent():ClearAllPoints()
			self:GetParent():SetPoint("LEFT", UIParent, "BOTTOMLEFT", -1, cursorY * GetScreenHeightScale())
		end
    end)
end

function menuPanelMixin:OnEvent(event, ...)
	if ( event == "ADDON_LOADED" ) then
		local name = ...
		if name == _ then
			self:CreateMenuPanel()
			if IsAddOnLoaded(_) then
				self:UnregisterEvent('ADDON_LOADED')
				self.ADDON_LOADED = nil
			end
		end
	elseif ( event == "GROUP_ROSTER_UPDATE" ) then
		self:OnGroupChange()
	end
end

function menuPanelMixin:OnLoad()
	for _, event in pairs({
		"ADDON_LOADED",
		"GROUP_ROSTER_UPDATE",
	}) do self:RegisterEvent(event) end
end